package com.qmth.distributed.print.api.admin;

import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.distributed.print.business.bean.params.SysAdminSetParam;
import com.qmth.distributed.print.business.service.ClientUpgradeService;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.enums.ClientUpgradeToolTypeEnum;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.clientpackage.ClientPackageEnum;
import com.qmth.teachcloud.common.util.Result;
import com.qmth.teachcloud.common.util.ResultUtil;
import com.qmth.teachcloud.data.entity.TSyncData;
import com.qmth.teachcloud.data.service.DataService;
import com.qmth.teachcloud.data.service.TSyncDataService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import java.util.Optional;

/**
 * 客户端升级 前端控制器
 */
@Api(tags = "超管-数据同步Controller")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + SystemConstant.PREFIX_URL_SYS + "/school/data")
@Aac(auth = false)
public class DataController {

    @Resource
    private DataService dataService;
    @Resource
    private TSyncDataService tSyncDataService;

    @ApiOperation(value = "测试连接")
    @RequestMapping(value = "/test/connect", method = RequestMethod.POST)
    public Result testConnect(@Valid @RequestBody SysAdminSetParam sysAdminSetParam, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultUtil.error(bindingResult.getAllErrors().get(0).getDefaultMessage());
        }
        Optional.ofNullable(sysAdminSetParam.getParam()).orElseThrow(() -> ExceptionResultEnum.PARAMS_ERROR.exception("数据同步参数未填写"));
        dataService.testConnect(sysAdminSetParam);
        return ResultUtil.ok(true);
    }

    @ApiOperation(value = "查看日志")
    @RequestMapping(value = "/param/list", method = RequestMethod.POST)
    public Result listParam(@ApiParam(value = "学校ID") @RequestParam(value = "schoolId") Long schoolId,
                            @ApiParam(value = "分页页码", required = true) @RequestParam @Min(SystemConstant.PAGE_NUMBER_MIN) Integer pageNumber,
                            @ApiParam(value = "分页数", required = true) @RequestParam @Min(SystemConstant.PAGE_SIZE_MIN) @Max(SystemConstant.PAGE_SIZE_MAX) Integer pageSize) {
        return ResultUtil.ok(tSyncDataService.listParam(schoolId, pageNumber, pageSize));
    }

    @ApiOperation(value = "新增/修改同步任务")
    @RequestMapping(value = "/param/save", method = RequestMethod.POST)
    public Result paramSave(@ApiParam(value = "任务对象") @RequestBody TSyncData tSyncData) {
        return ResultUtil.ok(tSyncDataService.saveData(tSyncData));
    }

    @ApiOperation(value = "手动同步")
    @RequestMapping(value = "/start/sync", method = RequestMethod.POST)
    public Result startSync(@ApiParam(value = "学校ID") @RequestParam(value = "schoolId") Long schoolId,
                            @ApiParam(value = "考试ID") @RequestParam(value = "examId") Long examId) {
        return ResultUtil.ok(tSyncDataService.syncData(schoolId, examId));
    }

    @ApiOperation(value = "查看日志")
    @RequestMapping(value = "/sync/log", method = RequestMethod.POST)
    public Result listLog(@ApiParam(value = "学校ID") @RequestParam(value = "schoolId") Long schoolId,
                          @ApiParam(value = "考试ID") @RequestParam(value = "examId") Long examId,
                          @ApiParam(value = "分页页码", required = true) @RequestParam @Min(SystemConstant.PAGE_NUMBER_MIN) Integer pageNumber,
                          @ApiParam(value = "分页数", required = true) @RequestParam @Min(SystemConstant.PAGE_SIZE_MIN) @Max(SystemConstant.PAGE_SIZE_MAX) Integer pageSize) {
        return ResultUtil.ok(tSyncDataService.getDataLog(schoolId, examId, pageNumber, pageSize));
    }

    @ApiOperation(value = "启用/禁用")
    @RequestMapping(value = "/enable/sync", method = RequestMethod.POST)
    public Result enable(@RequestBody TSyncData tSyncData) {
        return ResultUtil.ok(tSyncDataService.enable(tSyncData));
    }

}

