package com.qmth.teachcloud.data.service.impl;

import com.qmth.distributed.print.business.bean.params.SysAdminSetParam;
import com.qmth.teachcloud.common.entity.SysOrg;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.service.BasicCourseService;
import com.qmth.teachcloud.common.service.SysOrgService;
import com.qmth.teachcloud.common.service.SysUserService;
import com.qmth.teachcloud.data.DataUtil;
import com.qmth.teachcloud.data.entity.TSyncData;
import com.qmth.teachcloud.data.service.DataService;
import com.qmth.teachcloud.data.service.TSyncDataLogService;
import com.qmth.teachcloud.data.util.DruidJdbcUtils;
import com.qmth.teachcloud.data.util.JdbcUtil;
import com.qmth.teachcloud.data.vo.*;
import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.sql.DataSource;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.List;

@Service
public class DataServiceImpl implements DataService {

    @Resource
    private DataUtil dataUtil;
    @Resource
    private TSyncDataLogService tSyncDataLogService;
    @Resource
    private BasicCourseService basicCourseService;
    @Resource
    private SysOrgService sysOrgService;
    @Resource
    private SysUserService sysUserService;

    @Override
    public boolean testConnect(SysAdminSetParam sysAdminSetParam) {
        try {
            JdbcTemplate jdbcTemplate = JdbcUtil.getJdbcTemplate(dataUtil.createDataSourceMap(sysAdminSetParam.getParam()));
            String sql = "select 1 from dual";
            String object = jdbcTemplate.queryForObject(sql, String.class);
            if (StringUtils.isNotBlank(object)) {
                return true;
            } else {
                return false;
            }
        } catch (Exception e) {
            throw ExceptionResultEnum.ERROR.exception(e.getMessage());
        }
    }

    @Override
    public void syncData(Long schoolId, Long examId, TSyncData syncData) {
        JdbcTemplate jdbcTemplate = JdbcUtil.getJdbcTemplate(dataUtil.createDataSourceMap(schoolId));
        String dataRange = syncData.getDataType();
        // 机构同步
        if (dataRange.contains("A")) {
            long startTime = System.currentTimeMillis();
            List<OrgDataVo> dataVoList = dataUtil.listOrg(jdbcTemplate);
            int count = dataVoList.size();
            String errorMsg = null;
            try {
                for (OrgDataVo orgDataVo : dataVoList) {
                    sysOrgService.saveOrgFromSync(orgDataVo.transferToSysOrg(schoolId, orgDataVo));
                }
            } catch (Exception e) {
                count = 0;
                errorMsg = e.getMessage();
            } finally {
                long endTime = System.currentTimeMillis();
                tSyncDataLogService.saveLog(schoolId, examId, "A", startTime, endTime, count, errorMsg);
            }
        }
        // 用户同步
        if (dataRange.contains("B")) {
            long startTime = System.currentTimeMillis();
            List<UserDataVo> dataVoList = dataUtil.listUser(jdbcTemplate);
            int count = dataVoList.size();
            String errorMsg = null;
            try {
                for (UserDataVo userDataVo : dataVoList) {
                    sysUserService.saveUserFromSync(userDataVo.transferToUser(schoolId, userDataVo));
                }
            } catch (Exception e) {
                count = 0;
                errorMsg = e.getMessage();
            } finally {
                long endTime = System.currentTimeMillis();
                tSyncDataLogService.saveLog(schoolId, examId, "B", startTime, endTime, count, errorMsg);
            }
        }
        // 课程同步
        if (dataRange.contains("C")) {
            long startTime = System.currentTimeMillis();
            List<CourseDataVo> dataVoList = dataUtil.listCourse(jdbcTemplate);
            int count = dataVoList.size();
            String errorMsg = null;
            try {
                for (CourseDataVo courseDataVo : dataVoList) {
                    basicCourseService.saveBasicCourseFormSync(courseDataVo.transferToBasicCourse(schoolId, courseDataVo));
                }
            } catch (Exception e) {
                count = 0;
                errorMsg = e.getMessage();
            } finally {
                long endTime = System.currentTimeMillis();
                tSyncDataLogService.saveLog(schoolId, examId, "C", startTime, endTime, count, errorMsg);
            }
        }
        // 考生同步
        if (dataRange.contains("D")) {
            List<ExamStudentDataVo> dataVoList = dataUtil.listExamStudent(jdbcTemplate);
        }
        // 命题任务同步
        if (dataRange.contains("E")) {
            long startTime = System.currentTimeMillis();
            List<ExamTaskDataVo> dataVoList = dataUtil.listExamTask(jdbcTemplate);
            int count = dataVoList.size();
            String errorMsg = null;
            try {
                for (ExamTaskDataVo courseDataVo : dataVoList) {
//                    basicCourseService.saveBasicCourseFormSync(courseDataVo.transferToBasicCourse(schoolId, courseDataVo));
                }
            } catch (Exception e) {
                count = 0;
                errorMsg = e.getMessage();
            } finally {
                long endTime = System.currentTimeMillis();
                tSyncDataLogService.saveLog(schoolId, examId, "C", startTime, endTime, count, errorMsg);
            }
        }
    }
}
